# Import potřebných knihoven pro práci se sítí, soubory a geoprostorovými daty
import requests  # Pro stahování dat z internetu
from xml.etree import ElementTree as ET  # Pro parsování XML
from urllib.parse import urlparse  # Pro analýzu URL adres
import os  # Pro práci se souborovým systémem
import zipfile  # Pro práci se zip archivy
import arcpy  # Pro práci s ArcGIS Pro

# Nastavení Atom feedu a cesty pro stahování dat
feed_url = "https://atom.cuzk.cz/getservicefeed.ashx?service=TN-RAIL"
download_folder = r"C:\Users\lukas\Desktop\ArcGIS_Data"

# Nastavení cíle pro import dat do ArcGIS Pro
output_gdb = r"C:\Users\lukas\Documents\ArcGIS\Projects\DP\DP.gdb"

def download_zip_from_url(zip_url, download_path):
    """Stáhne zip soubor z dané URL a uloží ho na disk."""
    # Informace o začátku stahování
    print(f"Začíná stahování souboru: {os.path.basename(download_path)}")
    response = requests.get(zip_url, stream=True)  # Otevření spojení
    total_size = int(response.headers.get('content-length', 0))  # Celková velikost souboru
    downloaded_size = 0  # Inicializace stáhnouté velikosti

    with open(download_path, "wb") as file:
        for chunk in response.iter_content(chunk_size=8192):
            file.write(chunk)  # Zápis části dat do souboru
            downloaded_size += len(chunk)  # Aktualizace stáhnouté velikosti
            # Zobrazení postupu stahování
            if downloaded_size / total_size >= 0.01:
                percent_complete = int((downloaded_size / total_size) * 100)
                print(f"\rStahování {percent_complete}% dokončeno", end='')

    print(f"\nSoubor {os.path.basename(download_path)} byl úspěšně stažen.")

def sanitize_name(name):
    """Opraví název souboru tak, aby byl použitelný v ArcGIS."""
    invalid_chars = ' :;#&-.()!@%^*+=[]{}|\\,<>/?~'  # Seznam neplatných znaků
    for char in invalid_chars:
        name = name.replace(char, '_')  # Nahrazení neplatných znaků
    if name[0].isdigit():
        name = 'N' + name  # Přidání 'N' pokud název začíná číslem
    return name

# Vytvoření cílové složky pro stahování
os.makedirs(download_folder, exist_ok=True)

# Seznam pro uchování URL adres a cest zip souborů
zip_urls = []

# Stažení a parsování Atom feedu pro získání odkazů na data
response = requests.get(feed_url)
feed_xml = ET.fromstring(response.content)
namespaces = {'atom': 'http://www.w3.org/2005/Atom'}
dataset_feed_link = feed_xml.find('.//atom:link[@rel="alternate"][@title="dataset feed"]', namespaces)

if dataset_feed_link is not None:
    dataset_feed_url = dataset_feed_link.get('href')
    response = requests.get(dataset_feed_url)
    dataset_feed_xml = ET.fromstring(response.content)

    for entry in dataset_feed_xml.findall('.//atom:entry', namespaces):
        zip_url_element = entry.find('.//atom:link[@rel="alternate"][@type="application/geopackage+sqlite3"]', namespaces)
        if zip_url_element is not None:
            zip_url = zip_url_element.get('href')
            file_name = os.path.basename(urlparse(zip_url).path)
            download_path = os.path.join(download_folder, file_name)
            zip_urls.append((zip_url, download_path))

# Stažení všech nalezených souborů
for zip_url, download_path in zip_urls:
    download_zip_from_url(zip_url, download_path)

# Rozbalení stažených souborů
extract_paths = []
for _, download_path in zip_urls:
    file_name = os.path.basename(download_path)
    extract_path = os.path.join(download_folder, os.path.splitext(file_name)[0])
    if not os.path.exists(extract_path):
        os.makedirs(extract_path)
    with zipfile.ZipFile(download_path, 'r') as zip_ref:
        zip_ref.extractall(extract_path)
    extract_paths.append(extract_path)

# Import rozbalených dat do ArcGIS Pro
for extract_path in extract_paths:
    for root, dirs, files in os.walk(extract_path):
        for gpkg_file in files:
            if gpkg_file.endswith(".gpkg"):
                gpkg_path = os.path.join(root, gpkg_file)
                arcpy.env.workspace = gpkg_path  # Nastavení pracovního prostředí
                layers = arcpy.ListFeatureClasses()  # Seznam vrstev v gpkg
                for layer in layers:
                    layer_name = layer.split('.')[-1]  # Jméno vrstvy
                    out_layer_name = f"{os.path.basename(extract_path)}_{layer_name}"  # Nový název vrstvy
                    out_layer_name = sanitize_name(out_layer_name)  # Sanitace názvu
                    out_feature_class = os.path.join(output_gdb, out_layer_name)  # Cílová cesta
                    if arcpy.Exists(out_feature_class):
                        arcpy.Delete_management(out_feature_class)  # Smazání, pokud již existuje
                    arcpy.management.CopyFeatures(layer, out_feature_class)  # Import vrstvy
                    print(f"Importováno: {layer} do {out_feature_class}")
