import os  # Importuje modul pro interakci s operačním systémem
import re  # Importuje modul pro práci s regulárními výrazy
import arcpy  # Importuje modul pro práci s ArcGIS
from osgeo import ogr  # Importuje modul pro práci s geoprostorovými daty
from owslib.wfs import WebFeatureService  # Importuje třídu pro připojení k WFS službám

ogr.UseExceptions()  # Nastaví GDAL/OGR, aby vyvolával výjimky místo tišení chyb

# Definuje základní parametry pro přístup k WFS službě
wfs_url = "https://geoportal.cuzk.cz/wfs_inspire_tn_air/service.svc/get"  # URL WFS služby
wfs_version = "2.0.0"  # Verze WFS protokolu
output_format = 'text/xml; subtype=gml/3.2.1'  # Výstupní formát dat
wfs = WebFeatureService(url=wfs_url, version=wfs_version)  # Inicializuje objekt WFS

# Definuje cesty pro ukládání stahovaných dat a výstupní databáze
download_dir = 'C:/Users/lukas/Desktop/WFS_Downloads/'  # Adresář pro stahování souborů
output_gdb = 'C:/Users/lukas/Documents/ArcGIS/Projects/MyProject3/MyProject3.gdb/'  # Cesta k výstupní geodatabázi

# Zajišťuje existenci adresáře pro stahování dat
if not os.path.exists(download_dir):
    os.makedirs(download_dir)

# Funkce pro úpravu názvu vrstvy tak, aby byl validní pro ArcGIS (např. bez dvojteček a pomlček)
def clean_layer_name(layer_name):
    clean_name = layer_name.replace(':', '_').replace('-', '_')  # Nahradí nepovolené znaky v názvu
    if len(clean_name) > 60:  # Pokud je délka názvu delší než limit ArcGIS
        clean_name = clean_name[:60]  # Ořízne název na povolenou délku
    return clean_name

# Funkce pro extrakci EPSG kódu z dostupných možností CRS
def extract_epsg(crs_options):
    epsg_regex = re.compile(r"EPSG::(\d+)")  # Regulární výraz pro nalezení EPSG kódu
    for crs in crs_options:
        crs_str = str(crs)  # Převede CRS na řetězec
        match = epsg_regex.search(crs_str)  # Vyhledá shodu s regulárním výrazem
        if match:
            return "EPSG:" + match.group(1)  # Vrátí nalezený EPSG kód
    return "EPSG:5514"  # Výchozí hodnota pro Českou republiku

# Iteruje přes klíče ve WFS obsahu, které reprezentují jednotlivé vrstvy
for layer_name in list(wfs.contents.keys()):
    safe_layer_name = clean_layer_name(layer_name)  # Očistí název vrstvy
    layer_metadata = wfs.contents[layer_name]  # Získá metadata pro vrstvu
    crs = extract_epsg(layer_metadata.crsOptions)  # Získá EPSG kód souřadnicového systému
    print(f"Nastavovaný souřadnicový systém: {crs}")  # Vypíše nastavovaný souřadnicový systém
    
    # Definuje cestu, kam se uloží stažený GML soubor
    gml_file_path = os.path.join(download_dir, f"{safe_layer_name}.gml")
    # Definuje cestu, kam se uloží převedený GeoJSON soubor
    geojson_file_path = os.path.join(download_dir, f"{safe_layer_name}.geojson")
    # Definuje cestu, kam se v ArcGIS Pro uloží nově vytvořená Feature Class
    output_fc_path = os.path.join(output_gdb, safe_layer_name)

    # Vytvoření WFS dotazu pro stažení dat a zápis do GML souboru
    response = wfs.getfeature(typename=[layer_name], outputFormat=output_format)
    with open(gml_file_path, 'wb') as file:  # Otevře soubor pro binární zápis
        file.write(response.read())  # Čte odpověď ze serveru a zapisuje data do souboru

    # Otevře GML soubor pro konverzi do GeoJSON
    dataSource = ogr.Open(gml_file_path)
    daLayer = dataSource.GetLayer(0)  # Získá první vrstvu z datového zdroje
    driver = ogr.GetDriverByName('GeoJSON')  # Nastaví OGR driver pro GeoJSON
    if os.path.exists(geojson_file_path):  # Kontrola, zda již GeoJSON soubor existuje
        os.remove(geojson_file_path)  # Pokud ano, odstraní existující soubor
    outDataSource = driver.CreateDataSource(geojson_file_path)  # Vytvoří nový GeoJSON datový zdroj
    outLayer = outDataSource.CopyLayer(daLayer, str(1))  # Kopíruje vrstvu do nového GeoJSON souboru
    # Uvolní OGR datové zdroje z paměti
    del dataSource, outDataSource

    # Konverze GeoJSON souboru do Feature Class v ArcGIS
    arcpy.JSONToFeatures_conversion(geojson_file_path, output_fc_path)

    # Nastavení souřadnicového systému pro nově vytvořenou Feature Class podle extrahovaného EPSG kódu
    spatial_ref = arcpy.SpatialReference(int(crs.split(":")[1]))  # Vytvoří objekt souřadnicového systému
    arcpy.DefineProjection_management(output_fc_path, spatial_ref)  # Přiřadí souřadnicový systém k Feature Class

    # Vypíše zprávu o úspěšném zpracování vrstvy
    print(f"Vrstva {safe_layer_name} byla úspěšně zpracována a importována jako Feature Class: {output_fc_path}")

# Vypíše zprávu o dokončení zpracování všech vrstev
print("Zpracování všech vrstev dokončeno.")