import requests
import os
import zipfile
import arcpy
import time
from arcgis.gis import GIS
from arcgis.mapping import WebMap

def stahni_soubor(url_stahnuti, cilova_slozka):
    """
    Stahuje soubor z internetu do specifikované lokální složky.

    Args:
        url_stahnuti (str): URL adresa souboru k stahování.
        cilova_slozka (str): Cesta k lokální složce pro uložení souboru.
    """
    try:
        # Získání názvu souboru z URL
        nazev_souboru = os.path.basename(url_stahnuti)
        # Vytvoření cesty pro uložení souboru
        cesta_k_souboru = os.path.join(cilova_slozka, nazev_souboru)
        # Požadavek na server pro stahování souboru
        odpoved = requests.get(url_stahnuti, stream=True)
        # Získání celkové velikosti souboru z hlavičky odpovědi
        celkova_velikost = int(odpoved.headers.get('content-length', 0))
        # Inicializace proměnných pro sledování stahování
        stazeno = 0
        posledni_vypis_procent = 0
        
        # Vytvoření cílové složky, pokud neexistuje
        if not os.path.exists(cilova_slozka):
            os.makedirs(cilova_slozka)
        
        # Otevření souboru pro zápis binárních dat
        with open(cesta_k_souboru, 'wb') as soubor:
            for cast in odpoved.iter_content(chunk_size=1024):
                if odpoved.status_code == 200:
                    soubor.write(cast)
                    stazeno += len(cast)
                    # Výpočet a výpis pokroku stahování
                    procento = (stazeno / celkova_velikost) * 100
                    if procento - posledni_vypis_procent >= 1:
                        print(f"\rStahování... {procento:.0f}%", end="")
                        posledni_vypis_procent = procento
        print(f"\nSoubor {nazev_souboru} byl úspěšně stažen a uložen do {cilova_slozka}.")
    except Exception as e:
        print(f"Chyba při stahování souboru: {e}")

def rozbal_zip_soubory(cesta_k_slozce):
    """
    Rozbaluje všechny .zip soubory ve specifikované složce.

    Args:
        cesta_k_slozce (str): Cesta k složce obsahující .zip soubory.
    """
    # Procházení souborů ve složce
    for soubor in os.listdir(cesta_k_slozce):
        cesta_k_souboru = os.path.join(cesta_k_slozce, soubor)
        # Kontrola, jestli je soubor .zip
        if os.path.isfile(cesta_k_souboru) and soubor.endswith('.zip'):
            print(f"Rozbaluji soubor: {soubor}")
            try:
                # Otevření a rozbalení .zip souboru
                with zipfile.ZipFile(cesta_k_souboru, 'r') as zip_ref:
                    zip_ref.extractall(cesta_k_slozce)
                print(f"Soubor {soubor} byl úspěšně rozbalen.")
            except Exception as e:
                print(f"Chyba při rozbalování souboru {soubor}: {e}")

def odstran_vrstvy_a_soubory_gdb(cesta_gdb, seznam_vrstev_k_odstraneni):
    """
    Odstraňuje specifikované vrstvy a přidružené soubory z geodatabáze.

    Args:
        cesta_gdb (str): Cesta k geodatabázi, odkud se mají vrstvy odstranit.
        seznam_vrstev_k_odstraneni (list): Seznam názvů vrstev, které se mají odstranit.
    """
    # Iterace přes seznam vrstev určených k odstranění
    for nazev_vrstvy in seznam_vrstev_k_odstraneni:
        # Sestavení cesty k vrstvě v geodatabázi
        plna_cesta_k_vrstve = cesta_gdb + "\\" + nazev_vrstvy
        # Kontrola existence vrstvy a její odstranění
        if arcpy.Exists(plna_cesta_k_vrstve):
            arcpy.Delete_management(plna_cesta_k_vrstve)
            print(f"Vrstva/soubor '{nazev_vrstvy}' byla úspěšně odstraněna.")
        else:
            print(f"Vrstva/soubor '{nazev_vrstvy}' neexistuje.")

def importuj_shp_do_arcgis(cesta_projektu, cilove_pracoviste):
    """
    Importuje .shp soubory zadané cesty do projektu ArcGIS.

    Args:
        cesta_projektu (str): Cesta ke složce obsahující .shp soubory.
        cilove_pracoviste (str): Cílové umístění pro importované vrstvy.
    """
    # Nastavení pracovního prostředí a přepsání výstupu
    arcpy.env.workspace = cilove_pracoviste
    arcpy.env.overwriteOutput = True

    # Odstranění stávajících vrstev z aktuálně otevřené mapy
    mapa = arcpy.mp.ArcGISProject("CURRENT").listMaps()[0]
    for vrstva in mapa.listLayers():
        mapa.removeLayer(vrstva)

    # Procházení .shp souborů a jejich import do cílového pracoviště
    for root, dirs, files in os.walk(cesta_projektu):
        for soubor in files:
            if soubor.endswith('.shp'):
                cesta_k_shp = os.path.join(root, soubor)
                print(f"Importuji {cesta_k_shp} do {cilove_pracoviste}")
                nazev_vrstvy = os.path.splitext(soubor)[0].replace(" ", "_")
                try:
                    arcpy.FeatureClassToFeatureClass_conversion(cesta_k_shp, cilove_pracoviste, nazev_vrstvy)
                    print(f"Soubor {soubor} byl úspěšně importován jako {nazev_vrstvy}.")
                except Exception as e:
                    print(f"Chyba při importování souboru {soubor}: {e}")

def srovnej_vrstvy_podle_poradi(seznam_poradi):
    """
    Srovná vrstvy v ArcGIS projektu podle zadaného pořadí.

    Args:
        seznam_poradi (list): Seznam názvů vrstev v pořadí, ve kterém mají být srovnány.
    """
    # Získání odkazu na aktuální mapu v ArcGIS projektu
    mapa = arcpy.mp.ArcGISProject("CURRENT").listMaps()[0]
    # Srovnání vrstev podle zadaného seznamu
    for nazev_vrstvy in reversed(seznam_poradi):
        vrstvy = [v for v in mapa.listLayers() if v.name == nazev_vrstvy]
        if vrstvy:
            vrstva = vrstvy[0]
            ref_vrstva = mapa.listLayers()[0]
            print(f"Přesouvám {vrstva.name} před {ref_vrstva.name}")
            mapa.moveLayer(ref_vrstva, vrstva, "BEFORE")

def pridej_popisky_k_vrstvam():
    """
    Zapne popisky pro vybrané vrstvy v ArcGIS projektu.
    """
    # Získání odkazu na aktuální mapu
    mapa = arcpy.mp.ArcGISProject("CURRENT").listMaps()[0]
    vrstvy_pro_popisky = ["OKRESY_P", "VUSC_P", "STATY_P"]
    for vrstva in mapa.listLayers():
        if vrstva.name in vrstvy_pro_popisky and vrstva.supports("SHOWLABELS"):
            print(f"Zapínám popisky pro vrstvu: {vrstva.name}")
            vrstva.showLabels = True
            for lblClass in vrstva.listLabelClasses():
                lblClass.expression = "$feature.NAZEV"
                lblClass.visible = True
    print("Nastavení popisků dokončeno.")

def importuj_csv_jako_tabulku(csv_soubory, cilova_geodatabaze):
    """
    Importuje CSV soubory jako tabulky do specifikované geodatabáze.

    Args:
        csv_soubory (dict): Slovník, kde klíč je název cílové tabulky a hodnota je cesta k CSV souboru.
        cilova_geodatabaze (str): Cesta k cílové geodatabázi.
    """
    # Iterace přes slovník CSV souborů a jejich import
    for nazev_tabulky, cesta_k_csv in csv_soubory.items():
        vystupni_tabulka = f"{cilova_geodatabaze}/{nazev_tabulky}"
        # Kontrola existence tabulky a její případné odstranění
        if arcpy.Exists(vystupni_tabulka):
            print(f"Tabulka {nazev_tabulky} již existuje. Bude přepsána.")
            arcpy.Delete_management(vystupni_tabulka)
        # Import CSV souboru jako tabulky
        arcpy.TableToTable_conversion(cesta_k_csv, cilova_geodatabaze, nazev_tabulky)
        print(f"CSV soubor {cesta_k_csv} byl importován jako tabulka {nazev_tabulky}.")

def spoj_data_s_vrstvami(nazvy_vrstev, nazvy_tabulek, policko_pro_spojeni_vrstvy, policko_pro_spojeni_tabulky):
    """
    Spojuje data z tabulek s vrstvami v ArcGIS projektu na základě specifikovaného spojovacího pole.

    Args:
        nazvy_vrstev (list): Seznam názvů vrstev v projektu, které mají být spojeny s tabulkami.
        nazvy_tabulek (list): Seznam názvů tabulek, které mají být spojeny s vrstvami.
        policko_pro_spojeni_vrstvy (str): Název pole ve vrstvě, podle kterého se provádí spojení.
        policko_pro_spojeni_tabulky (str): Název pole v tabulce, podle kterého se provádí spojení.
    """
    # Získání odkazu na aktuální mapu
    mapa = arcpy.mp.ArcGISProject("CURRENT").listMaps()[0]
    # Procházení vrstev a jejich spojování s odpovídajícími tabulkami
    for nazev_vrstvy, nazev_tabulky in zip(nazvy_vrstev, nazvy_tabulek):
        # Získání vrstvy podle názvu
        vrstva = next((v for v in mapa.listLayers() if v.name == nazev_vrstvy), None)
        if vrstva:
            # Vytvoření dočasného pohledu na tabulku pro spojení
            nazev_pohledu_tabulky = f"{nazev_tabulky}_view"
            arcpy.MakeTableView_management(nazev_tabulky, nazev_pohledu_tabulky)
            # Přidání spojení mezi vrstvou a tabulkou
            arcpy.AddJoin_management(vrstva, policko_pro_spojeni_vrstvy, nazev_pohledu_tabulky, policko_pro_spojeni_tabulky, "KEEP_ALL")
            print(f"Data z tabulky '{nazev_tabulky}' byla spojena s vrstvou '{nazev_vrstvy}'.")
            # Odstranění dočasného pohledu na tabulku po spojení
            arcpy.Delete_management(nazev_pohledu_tabulky)

# Definice hlavní funkce, která řídí celý proces skriptu
def hlavni():
    """
    Hlavní funkce, která řídí celý proces skriptu od stahování a rozbalení dat,
    přes import dat do ArcGIS, až po jejich publikování na ArcGIS Online.
    """
    # URL adresa pro stahování ZIP souboru s daty
    url_stahnuti = "https://services.cuzk.cz/shp/stat/epsg-5514/1.zip"
    # Cesta, kam se mají data stáhnout a rozbalit
    cilova_slozka_stahnuti = "C:/Users/lukas/Documents/ArcGIS/Projects/DP_Final_script/Download"
    
    # Stahování a rozbalení ZIP souboru
    stahni_soubor(url_stahnuti, cilova_slozka_stahnuti)
    rozbal_zip_soubory(cilova_slozka_stahnuti)

    # Cesta k projektu a cílové geodatabázi pro import dat
    cesta_projektu = cilova_slozka_stahnuti
    cilova_gdb = "C:/Users/lukas/Documents/ArcGIS/Projects/DP_Final_script/DP_Final_script.gdb"
    
    # Import SHP souborů do ArcGIS
    importuj_shp_do_arcgis(cesta_projektu, cilova_gdb)

        # Seznam vrstev k odstranění před importem nových dat
    vrstvy_k_odstraneni = [
        "ZSJ_P",
        "VO_P",
        "STU_P",
        "REGION_P",
        "PRARES_P",
        "POU_P",
        "ORP_P",
        "OBCE_P",
        "KATUZE_P"
    ]

    # Odstranění vrstev a souborů GDB před importem nových dat
    odstran_vrstvy_a_soubory_gdb(cilova_gdb, vrstvy_k_odstraneni)

    # Seznam vrstev pro uspořádání v mapě
    seznam_poradi_vrstev = ["OKRESY_P", "VUSC_P", "STATY_P"]
    # Uspořádání vrstev v mapě podle zadaného seznamu
    srovnej_vrstvy_podle_poradi(seznam_poradi_vrstev)

    # Přidání popisků k vrstvám v mapě
    pridej_popisky_k_vrstvam()
    
    # Seznam CSV souborů pro import a jejich cesty
    csv_soubory_pro_import = {
        "o_cr": "C:/Users/lukas/Documents/ArcGIS/Projects/DP_Final_script/Data_csv/o-cr.csv",
        "o_kraje": "C:/Users/lukas/Documents/ArcGIS/Projects/DP_Final_script/Data_csv/o-kraje.csv",
        "o_okresy": "C:/Users/lukas/Documents/ArcGIS/Projects/DP_Final_script/Data_csv/o-okresy.csv"
    }
    # Import CSV souborů jako tabulek do geodatabáze
    importuj_csv_jako_tabulku(csv_soubory_pro_import, cilova_gdb)

    # Spojuje data s vrstvami na základě názvu území
    spoj_data_s_vrstvami(["OKRESY_P", "VUSC_P", "STATY_P"], ["o_okresy", "o_kraje", "o_cr"], "NAZEV", "Území")
    
    # Cesta k lokální geodatabázi, kterou chceme zabalit a nahrát
    cesta_k_geodatabazi = r"C:\Users\lukas\Documents\ArcGIS\Projects\DP_Final_script\DP_Final_script.gdb"
    cesta_k_zip = cesta_k_geodatabazi + ".zip"
    # Zdržení na 30 sekund pro dokončení předchozích operací
    time.sleep(30)

    # Zabalí geodatabázi do ZIP souboru, přeskočí .lock soubory
    with zipfile.ZipFile(cesta_k_zip, 'w', zipfile.ZIP_DEFLATED) as zip_soubor:
        for koren, slozky, soubory in os.walk(cesta_k_geodatabazi):
            for soubor in soubory:
                if not soubor.endswith('.lock'):  # Přeskočit .lock soubory
                    plna_cesta_souboru = os.path.join(koren, soubor)
                    relativni_cesta_souboru = os.path.relpath(plna_cesta_souboru, os.path.join(cesta_k_geodatabazi, '..'))
                    zip_soubor.write(plna_cesta_souboru, relativni_cesta_souboru)

    # Přihlášení do ArcGIS Online
    gis = GIS("home")

    # Nahrání a publikování ZIP souboru jako feature služby
    polozka_data = gis.content.add({"title": "DP_Final_script GDB", "tags": "GDB, Upload"}, data=cesta_k_zip)
    publikovana_polozka = polozka_data.publish()

    print(f"Vrstva byla úspěšně publikována: {publikovana_polozka.url}")

    # Vytvoření nové webové mapy
    webova_mapa = WebMap()

    # Přidání všech publikovaných vrstev do webové mapy
    for vrstva in publikovana_polozka.layers:
        webova_mapa.add_layer(vrstva)

    # Nastavení vlastností webové mapy
    vlastnosti_webove_mapy = {
        'title': 'DP - Final_script',
        'snippet': 'Mapa zahrnující data z projektu DP_Final_script.',
        'tags': 'DP, ArcGIS Pro'
    }

    # Uložení webové mapy na ArcGIS Online bez specifikace složky
    polozka_webove_mapy = webova_mapa.save(item_properties=vlastnosti_webove_mapy)

    print(f"Webová mapa byla úspěšně vytvořena a uložena: {polozka_webove_mapy.homepage}")

if __name__ == "__main__":
    # Spustí hlavní funkci skriptu, pokud je soubor spuštěn přímo.
    # To zajišťuje, že pokud je tento skript importován jako modul v jiném souboru,
    # nespustí se automaticky hlavní logika, což umožňuje jeho použití a testování
    # v širším kontextu bez nežádoucího spuštění celého procesu.
    hlavni()


